# SPDX-License-Identifier: GPL-2.0-or-later

HIRTE_DEFAULT_PORT = 842
HIRTE_DEFAULT_HOST = "127.0.0.1"

HIRTE_DBUS_INTERFACE = "org.containers.hirte"
HIRTE_OBJECT_PATH = "/org/containers/hirte"
HIRTE_AGENT_DBUS_INTERFACE = "org.containers.hirte.Agent"
